#!/usr/bin/env ruby
require "fileutils"
require 'pp'
$LOAD_PATH << "/usr/lib64/ruby/site_ruby/1.8/x86_64-linux"
require 'RRD'
require 'yaml'

SCRIPT_DIR=File.dirname(File.expand_path(__FILE__))
RRD_DIR="#{SCRIPT_DIR}/rrd"
PAST_DATA="#{SCRIPT_DIR}/past_data.yml"
IPVSADM="/sbin/ipvsadm"
CMD_STATS="#{IPVSADM} -L -n --stats --exact"
CMD_CONN="#{IPVSADM} -L -n"
LOG_FILE="/var/log/lvs_stats.log"
$negative_validated=false

$log = File.open(LOG_FILE, "a") if ARGV[0] == "log"

def rrd_stats_create(rrd_name)
  RRD.create(
    rrd_name,
   "--start", "#{Time.now.to_i-1}",
   "--step",  "300",
   "DS:ActiveConn:GAUGE:600:U:U",
   "DS:InActConn:GAUGE:600:U:U",
   "DS:Conns:COUNTER:600:U:U",
   "DS:InPkts:COUNTER:600:U:U",
   "DS:OutPkts:COUNTER:600:U:U",
   "DS:InBytes:COUNTER:600:U:U",
   "DS:OutBytes:COUNTER:600:U:U",
   "RRA:AVERAGE:0.5:1:28800",
   "RRA:AVERAGE:0.5:6:4800",
   "RRA:AVERAGE:0.5:24:2400",
   "RRA:AVERAGE:0.5:288:797",
   "RRA:MAX:0.5:1:28800",
   "RRA:MAX:0.5:6:4800",
   "RRA:MAX:0.5:24:2400",
   "RRA:MAX:0.5:288:797"
  )
end

def validate_negative_value(stats_table)
  past_table = Hash.new
  if File.exist?(PAST_DATA)
    past_table = YAML.load(File.read(PAST_DATA))
    deleted_vs = past_table.keys - stats_table.keys
    past_table.delete_if { |vs_name, val| deleted_vs.include?(vs_name) }
    $negative_validated = true
  end

  stats_table.keys.each do |vs_name|
    delta=[]
    past = past_table[vs_name]
    now  = stats_table[vs_name]
    if past
      now.zip(past) { |v_now, v_past| delta << v_now.to_i - v_past.to_i }
      stats_table[vs_name].each_with_index do |e,idx|
        if delta[idx] < 0
          $log.puts "WARN:#{TIME}  #{vs_name} #{idx} #{stats_table[vs_name][idx]}" if $log
          stats_table[vs_name][idx] = "U"
        end
      end
    end
    past_table[vs_name] = now
  end
  File.open(PAST_DATA, "w") do |io|
    io.puts YAML.dump(past_table)
  end
  return stats_table
end

vs_name=""

conn_table=Hash.new{|h,k| h[k] = [0,0] }
stats_table=Hash.new{|h,k| h[k] = [0,0,0,0,0] }

conn_result=`#{CMD_CONN}`
conn_result.each do |line|
  if line =~ /^TCP/
    vs_name=line.split[1]
  elsif line =~ /\s+->\s+/
    next if vs_name.empty?
    #pp line.split.values_at(-2,-1)
    act_conn , inact_conn = line.split.values_at(-2,-1)
    conn_table[vs_name][0] += act_conn.to_i
    conn_table[vs_name][1] += inact_conn.to_i
  end
end

#puts conn_table.keys

stats_result=`#{CMD_STATS}`


TIME=Time.now.strftime("%Y%m%d-%H%M%S")
#if $log
#  #$log.puts "==== #{Time.now.strftime("%Y%m%d-%H%M%S")} ===="
#  #$log.puts conn_result
#  #$log.puts
#  #$log.puts stats_result
#  #$log.puts
#end


stats_result.each do |line|
  line.chomp!
  if line =~ /^TCP/
    vs_name, conns, in_pkts, out_pkts, in_bytes, out_bytes= line.split[1..-1]
    rrd="#{RRD_DIR}/#{vs_name.sub(":","_")}.rrd"
    unless FileTest.exist?(rrd)
      rrd_stats_create(rrd)
      #puts %Q!rrd_stats_create(#{rrd})!
    end
    stats_table[vs_name] = conns, in_pkts, out_pkts, in_bytes, out_bytes
  end
end

stats_table = validate_negative_value(stats_table)

stats_table.each do |vs_name, stats_count|
  conn_str = conn_table[vs_name].join(":")
  update_str = "N:#{conn_str}:#{stats_count.join(":")}"
  rrd="#{RRD_DIR}/#{vs_name.sub(":","_")}.rrd"
  #$log.puts %Q!RRD.update(#{rrd},"#{update_str}")! if $log
  RRD.update(rrd,"#{update_str}") if $negative_validated
end
$log.close if $log
